/*
 * Copyright (C) 2011 Atmark Techno, Inc. All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/serial.h>
#include <linux/gpio.h>
#include <linux/regulator/consumer.h>
#include <mach/hardware.h>
#include <mach/mxc_uart.h>
#include "armadillo400_iomux.h"
#include "armadillo400_serial.h"
#include "board.h"

static struct pad_desc armadillo400_uart_pad =
	MX25_PAD_CTL_GRP_DSE_UART(PAD_CTL_DSE_LOW);

static struct pad_desc armadillo400_uart2_pads[] = {
	MX25_PAD_UART2_RXD__UART2_RXD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_UART2_TXD__UART2_TXD(0),
	MX25_PAD_UART2_RTS__UART2_RTS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_UART2_CTS__UART2_CTS(0),
	MX25_PAD_UART1_RXD__UART2_DTR(0),
	MX25_PAD_UART1_TXD__UART2_DSR(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_UART1_RTS__UART2_DCD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_UART1_CTS__UART2_RI(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	/* UART2_FORCEOFF */
	MX25_PAD_BOOT_MODE1__GPIO_4_31(),
};

static struct pad_desc armadillo400_uart3_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON9_3_UART3_RXD)
	MX25_PAD_CSPI1_MOSI__UART3_RXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_40_UART3_RXD)
	MX25_PAD_KPP_ROW0__UART3_RXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_5_UART3_TXD)
	MX25_PAD_CSPI1_MISO__UART3_TXD(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_41_UART3_TXD)
	MX25_PAD_KPP_ROW1__UART3_TXD(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_11_UART3_RTS)
	MX25_PAD_CSPI1_SS1__UART3_RTS(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_42_UART3_RTS)
	MX25_PAD_KPP_ROW2__UART3_RTS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_13_UART3_CTS)
	MX25_PAD_CSPI1_SCLK__UART3_CTS(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_43_UART3_CTS)
	MX25_PAD_KPP_ROW3__UART3_CTS(0),
#endif
};

static struct pad_desc armadillo400_uart4_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON11_44_UART4_RXD)
	MX25_PAD_KPP_COL0__UART4_RXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_45_UART4_TXD)
	MX25_PAD_KPP_COL1__UART4_TXD(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_46_UART4_RTS)
	MX25_PAD_KPP_COL2__UART4_RTS(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_47_UART4_CTS)
	MX25_PAD_KPP_COL3__UART4_CTS(0),
#endif
};

static struct pad_desc armadillo400_uart5_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON9_4_UART5_RXD)
	MX25_PAD_CSI_D2__UART5_RXD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_6_UART5_TXD)
	MX25_PAD_CSI_D3__UART5_TXD(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_12_UART5_RTS)
	MX25_PAD_CSI_D4__UART5_RTS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_14_UART5_CTS)
	MX25_PAD_CSI_D5__UART5_CTS(0),
#endif
};

static char *uart_regulator_name[5] = {
	"uart1-power", "uart2-power", "uart3-power", "uart4-power",
	"uart5-power"
};
static struct regulator *uart_regulator[5] = { NULL, NULL, NULL, NULL, NULL };
static int activated_port = 0;

static int platform_uart_init(struct device *dev)
{
	struct platform_device *pdev = to_platform_device(dev);
	uart_mxc_port *uart = dev->platform_data;
	int port = pdev->id;

	if (!uart->ir_mode)
		return -ENOSYS;

	mxc_iomux_v3_setup_pad(&armadillo400_uart_pad);

	switch (port) {
	case 0:
		break;
	case 1:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_uart2_pads);
		if (!(activated_port & (1 << port)))
			gpio_request(UART2_FORCEOFF_GPIO, "UART2_FORCEOFF");
		gpio_direction_output(UART2_FORCEOFF_GPIO, 1);
		break;
	case 2:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_uart3_pads);
		break;
	case 3:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_uart4_pads);
		break;
	case 4:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_uart5_pads);
		break;
	default:
		return -ENODEV;
	}

	if (!uart_regulator[port]) {
		uart_regulator[port] = regulator_get(dev,
						     uart_regulator_name[port]);
		if (IS_ERR(uart_regulator[port]))
			uart_regulator[port] = NULL;
	}
	if (uart_regulator[port])
		regulator_enable(uart_regulator[port]);

	activated_port |= (1 << port);

	return 0;
}

static int platform_uart_exit(struct device *dev)
{
	struct platform_device *pdev = to_platform_device(dev);
	int port = pdev->id;

	switch (port) {
	case 0:
		break;
	case 1:
		if (activated_port & (1 << port))
			gpio_set_value(UART2_FORCEOFF_GPIO, 0);
		break;
	case 2:
		break;
	case 3:
		break;
	case 4:
		break;
	default:
		return -ENODEV;
	}

	if (uart_regulator[port])
		regulator_disable(uart_regulator[port]);

	return 0;
}

/*
 * This is an array where each element holds information about a UART port,
 * like base address of the UART, interrupt numbers etc. This structure is
 * passed to the serial_core.c file. Based on which UART is used, the core file
 * passes back the appropriate port structure as an argument to the control
 * functions.
 */
static uart_mxc_port mxc_ports[] = {
#if UART1_ENABLED == 1
	[0] = {
		.port = {
			.iotype = SERIAL_IO_MEM,
			.fifosize = 32,
			.flags = UPF_BOOT_AUTOCONF,
			.line = 0,
		},
		.ints_muxed = UART1_MUX_INTS,
		.mode = UART1_MODE,
		.ir_mode = UART1_IR,
		.enabled = UART1_ENABLED,
		.hardware_flow = UART1_HW_FLOW,
		.cts_threshold = UART1_UCR4_CTSTL,
		.dma_enabled = UART1_DMA_ENABLE,
		.dma_rxbuf_size = UART1_DMA_RXBUFSIZE,
		.rx_threshold = UART1_UFCR_RXTL,
		.tx_threshold = UART1_UFCR_TXTL,
		.dma_tx_id = MXC_DMA_UART1_TX,
		.dma_rx_id = MXC_DMA_UART1_RX,
		.rxd_mux = MXC_UART_RXDMUX,

		.init = platform_uart_init,
		.exit = platform_uart_exit,
#if defined(CONFIG_ARMADILLO400_UART1_WAKE_SRC_SELECT)
		.should_wakeup = 1,
#endif
	},
#endif
#if UART2_ENABLED == 1
	[1] = {
		.port = {
			.iotype = SERIAL_IO_MEM,
			.fifosize = 32,
			.flags = UPF_BOOT_AUTOCONF,
			.line = 1,
		},
		.ints_muxed = UART2_MUX_INTS,
		.mode = UART2_MODE,
		.ir_mode = UART2_IR,
		.enabled = UART2_ENABLED,
		.hardware_flow = UART2_HW_FLOW,
		.cts_threshold = UART2_UCR4_CTSTL,
		.dma_enabled = UART2_DMA_ENABLE,
		.dma_rxbuf_size = UART2_DMA_RXBUFSIZE,
		.rx_threshold = UART2_UFCR_RXTL,
		.tx_threshold = UART2_UFCR_TXTL,
		.dma_tx_id = MXC_DMA_UART2_TX,
		.dma_rx_id = MXC_DMA_UART2_RX,
		.rxd_mux = MXC_UART_IR_RXDMUX,

		.init = platform_uart_init,
		.exit = platform_uart_exit,
#if defined(CONFIG_ARMADILLO400_UART2_WAKE_SRC_SELECT)
		.should_wakeup = 1,
#endif
	},
#endif
#if UART3_ENABLED == 1
	[2] = {
		.port = {
			.iotype = SERIAL_IO_MEM,
			.fifosize = 32,
			.flags = UPF_BOOT_AUTOCONF,
			.line = 2,
		},
		.ints_muxed = UART3_MUX_INTS,
		.mode = UART3_MODE,
		.ir_mode = UART3_IR,
		.enabled = UART3_ENABLED,
		.hardware_flow = UART3_HW_FLOW,
		.cts_threshold = UART3_UCR4_CTSTL,
		.dma_enabled = UART3_DMA_ENABLE,
		.dma_rxbuf_size = UART3_DMA_RXBUFSIZE,
		.rx_threshold = UART3_UFCR_RXTL,
		.tx_threshold = UART3_UFCR_TXTL,
		.dma_tx_id = MXC_DMA_UART3_TX,
		.dma_rx_id = MXC_DMA_UART3_RX,
		.rxd_mux = MXC_UART_RXDMUX,

		.init = platform_uart_init,
		.exit = platform_uart_exit,
#if defined(CONFIG_ARMADILLO400_UART3_WAKE_SRC_SELECT)
		.should_wakeup = 1,
#endif
	},
#endif
#if UART4_ENABLED == 1
	[3] = {
		.port = {
			.iotype = SERIAL_IO_MEM,
			.fifosize = 32,
			.flags = UPF_BOOT_AUTOCONF,
			.line = 3,
		},
		.ints_muxed = UART4_MUX_INTS,
		.mode = UART4_MODE,
		.ir_mode = UART4_IR,
		.enabled = UART4_ENABLED,
		.hardware_flow = UART4_HW_FLOW,
		.cts_threshold = UART4_UCR4_CTSTL,
		.dma_enabled = UART4_DMA_ENABLE,
		.dma_rxbuf_size = UART4_DMA_RXBUFSIZE,
		.rx_threshold = UART4_UFCR_RXTL,
		.tx_threshold = UART4_UFCR_TXTL,
		.dma_tx_id = MXC_DMA_UART4_TX,
		.dma_rx_id = MXC_DMA_UART4_RX,
		.rxd_mux = MXC_UART_RXDMUX,

		.init = platform_uart_init,
		.exit = platform_uart_exit,
#if defined(CONFIG_ARMADILLO400_UART4_WAKE_SRC_SELECT)
		.should_wakeup = 1,
#endif
	},
#endif
#if UART5_ENABLED == 1
	[4] = {
		.port = {
			.iotype = SERIAL_IO_MEM,
			.fifosize = 32,
			.flags = UPF_BOOT_AUTOCONF,
			.line = 4,
		},
		.ints_muxed = UART5_MUX_INTS,
		.mode = UART5_MODE,
		.ir_mode = UART5_IR,
		.enabled = UART5_ENABLED,
		.hardware_flow = UART5_HW_FLOW,
		.cts_threshold = UART5_UCR4_CTSTL,
		.dma_enabled = UART5_DMA_ENABLE,
		.dma_rxbuf_size = UART5_DMA_RXBUFSIZE,
		.rx_threshold = UART5_UFCR_RXTL,
		.tx_threshold = UART5_UFCR_TXTL,
		.dma_tx_id = MXC_DMA_UART5_TX,
		.dma_rx_id = MXC_DMA_UART5_RX,
		.rxd_mux = MXC_UART_RXDMUX,

		.init = platform_uart_init,
		.exit = platform_uart_exit,
#if defined(CONFIG_ARMADILLO400_UART5_WAKE_SRC_SELECT)
		.should_wakeup = 1,
#endif
	},
#endif
};

#if UART1_ENABLED == 1
static struct resource mxc_uart_resources1[] = {
	{
		.start = MX25_UART1_BASE_ADDR,
		.end = MX25_UART1_BASE_ADDR + 0x0B5,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = UART1_INT1,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART1_INT2,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART1_INT3,
		.flags = IORESOURCE_IRQ,
	},

};

static struct platform_device mxc_uart_device1 = {
	.name = "mxcintuart",
	.id = 0,
	.num_resources = ARRAY_SIZE(mxc_uart_resources1),
	.resource = mxc_uart_resources1,
	.dev = {
		.platform_data = &mxc_ports[0],
	},
};
#endif
#if UART2_ENABLED == 1
static struct resource mxc_uart_resources2[] = {
	{
		.start = MX25_UART2_BASE_ADDR,
		.end = MX25_UART2_BASE_ADDR + 0x0B5,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = UART2_INT1,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART2_INT2,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART2_INT3,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device mxc_uart_device2 = {
	.name = "mxcintuart",
	.id = 1,
	.num_resources = ARRAY_SIZE(mxc_uart_resources2),
	.resource = mxc_uart_resources2,
	.dev = {
		.platform_data = &mxc_ports[1],
	},
};
#endif
#if UART3_ENABLED == 1
static struct resource mxc_uart_resources3[] = {
	{
		.start = MX25_UART3_BASE_ADDR,
		.end = MX25_UART3_BASE_ADDR + 0x0B5,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = UART3_INT1,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART3_INT2,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART3_INT3,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device mxc_uart_device3 = {
	.name = "mxcintuart",
	.id = 2,
	.num_resources = ARRAY_SIZE(mxc_uart_resources3),
	.resource = mxc_uart_resources3,
	.dev = {
		.platform_data = &mxc_ports[2],
	},
};
#endif
#if UART4_ENABLED == 1
static struct resource mxc_uart_resources4[] = {
	{
		.start = MX25_UART4_BASE_ADDR,
		.end = MX25_UART4_BASE_ADDR + 0x0B5,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = UART4_INT1,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART4_INT2,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART4_INT3,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device mxc_uart_device4 = {
	.name = "mxcintuart",
	.id = 3,
	.num_resources = ARRAY_SIZE(mxc_uart_resources4),
	.resource = mxc_uart_resources4,
	.dev = {
		.platform_data = &mxc_ports[3],
	},
};
#endif
#if UART5_ENABLED == 1
static struct resource mxc_uart_resources5[] = {
	{
		.start = MX25_UART5_BASE_ADDR,
		.end = MX25_UART5_BASE_ADDR + 0x0B5,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = UART5_INT1,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART5_INT2,
		.flags = IORESOURCE_IRQ,
	},
	{
		.start = UART5_INT3,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device mxc_uart_device5 = {
	.name = "mxcintuart",
	.id = 4,
	.num_resources = ARRAY_SIZE(mxc_uart_resources5),
	.resource = mxc_uart_resources5,
	.dev = {
		.platform_data = &mxc_ports[4],
	},
};
#endif

void __init armadillo400_uart_init(void)
{
	/* Register all the MXC UART platform device structures */
#if UART1_ENABLED == 1
	platform_device_register(&mxc_uart_device1);
#endif
#if UART2_ENABLED == 1
	platform_device_register(&mxc_uart_device2);
#endif
#if UART3_ENABLED == 1
	platform_device_register(&mxc_uart_device3);
#endif
#if UART4_ENABLED == 1
	platform_device_register(&mxc_uart_device4);
#endif
#if UART5_ENABLED == 1
	platform_device_register(&mxc_uart_device5);
#endif
}
